@@ -154,7 +154,8 @@ class CodeActAgent:
             current_date = datetime.now().strftime("%Y-%m-%d")
             system_prompt = self.prompts.system.render(tools=tools, current_date=current_date)
 
-            messages = [ChatMessage(role="system", content=system_prompt)] + messages
+            if messages and messages[0].role not in ("system", "developer"):
+                messages = [ChatMessage(role="system", content=system_prompt)] + messages
 
             for step_number in range(1, self.max_iterations + 1):
                 # Optional planning step
@@ -162,7 +163,9 @@ class CodeActAgent:
                     step_number == 1 or (step_number - 1) % self.planning_interval == 0
                 ):
                     self._log(
-                        f"Planning step {step_number} started", run_id=run_id, session_id=session_id
+                        f"Planning step {step_number} started",
+                        run_id=run_id,
+                        session_id=session_id,
                     )
                     new_messages = await self._run_planning_step(
                         messages=messages,
@@ -179,16 +182,25 @@ class CodeActAgent:
                     )
 
                 # Main step
-                self._log(f"Step {step_number} started", run_id=run_id, session_id=session_id)
+                self._log(
+                    f"Step {step_number} started",
+                    run_id=run_id,
+                    session_id=session_id,
+                )
                 new_messages = await self._step(
                     messages,
                     python_executor=python_executor,
                     session_id=session_id,
                     run_id=run_id,
                     event_bus=event_bus,
+                    step_number=step_number,
                 )
                 messages.extend(new_messages)
-                self._log(f"Step {step_number} completed", run_id=run_id, session_id=session_id)
+                self._log(
+                    f"Step {step_number} completed",
+                    run_id=run_id,
+                    session_id=session_id,
+                )
                 if messages[-1].role == "assistant":
                     break
             else:
@@ -197,7 +209,11 @@ class CodeActAgent:
                     messages, session_id=session_id, run_id=run_id, event_bus=event_bus
                 )
                 messages.extend(new_messages)
-                self._log("Final step completed", run_id=run_id, session_id=session_id)
+                self._log(
+                    "Final step completed",
+                    run_id=run_id,
+                    session_id=session_id,
+                )
 
         except Exception as e:
             self._log(
@@ -245,7 +261,9 @@ class CodeActAgent:
         session_id: str,
         run_id: str,
         event_bus: AgentEventBus | None = None,
+        step_number: int | None = None,
     ) -> ChatMessages:
+        _ = step_number  # for logs only
         self._log(
             f"Step inputs: {messages}", run_id=run_id, session_id=session_id, level=logging.DEBUG
         )
@@ -397,9 +415,11 @@ class CodeActAgent:
         last_n: int = PLANNING_LAST_N,
         content_max_length: int = PLANNING_CONTENT_MAX_LENGTH,
     ) -> str:
-        def messages_to_string(messages: ChatMessages) -> str:
+        messages = copy.deepcopy(messages)
+
+        def messages_to_string(messages_internal: ChatMessages) -> str:
             str_messages = []
-            for m in messages:
+            for m in messages_internal:
                 content = truncate_content(str(m.content), max_length=content_max_length)
                 str_messages.append(f"{m.role}: {content}")
             return "\n\n".join(str_messages)
@@ -420,8 +440,7 @@ class CodeActAgent:
             return ""
         if len(used_messages) <= last_n:
             return messages_to_string(used_messages)
-        used_messages = used_messages[-last_n:]
-        conversation = messages_to_string(used_messages)
+        conversation = messages_to_string(used_messages[-last_n:])
         first_message = messages_to_string(used_messages[:1])
         return f"First message:\n\n{first_message}\n\nLast {last_n} messages:\n\n{conversation}"
 
@@ -433,6 +452,7 @@ class CodeActAgent:
         session_id: str,
         event_bus: AgentEventBus | None = None,
     ) -> ChatMessages:
+        messages = copy.deepcopy(messages)
         assert self.prompts.plan is not None, "Planning prompt is not set, but planning is enabled"
         assert (
             self.prompts.plan_prefix is not None
@@ -499,5 +519,6 @@ class CodeActAgent:
         )
 
     def _log(self, message: str, run_id: str, session_id: str, level: int = logging.INFO) -> None:
-        message = f"| {session_id:<8} | {run_id:<8} | {message}"
+        timestamp = datetime.now().strftime("%Y-%m-%d %H:%M:%S")
+        message = f"| {timestamp} | {session_id:<8} | {run_id:<8} | {message}"
         self.logger.log(level, message)
